@@ -1,32 +1,27 @@
 """
-The axes_divider module provides helper classes to adjust the positions of
-multiple axes at drawing time.
-
- Divider: this is the class that is used to calculate the axes
-    position. It divides the given rectangular area into several sub
-    rectangles. You initialize the divider by setting the horizontal
-    and vertical lists of sizes that the division will be based on. You
-    then use the new_locator method, whose return value is a callable
-    object that can be used to set the axes_locator of the axes.
+Helper classes to adjust the positions of multiple axes at drawing time.
 """
 
-from matplotlib import cbook
+import numpy as np
+
+import matplotlib as mpl
+from matplotlib import _api
 from matplotlib.axes import SubplotBase
-from matplotlib.gridspec import SubplotSpec, GridSpec
+from matplotlib.gridspec import SubplotSpec
 import matplotlib.transforms as mtransforms
 from . import axes_size as Size
 
 
 class Divider:
     """
-    This class calculates the axes position. It
-    divides the given rectangular area into several
-    sub-rectangles. You initialize the divider by setting the
-    horizontal and vertical lists of sizes
-    (:mod:`mpl_toolkits.axes_grid.axes_size`) that the division will
-    be based on. You then use the new_locator method to create a
-    callable object that can be used as the axes_locator of the
-    axes.
+    An Axes positioning class.
+
+    The divider is initialized with lists of horizontal and vertical sizes
+    (:mod:`mpl_toolkits.axes_grid1.axes_size`) based on which a given
+    rectangular area will be divided.
+
+    The `new_locator` method then creates a callable object
+    that can be used as the *axes_locator* of the axes.
     """
 
     def __init__(self, fig, pos, horizontal, vertical,
@@ -36,17 +31,17 @@ class Divider:
         ----------
         fig : Figure
         pos : tuple of 4 floats
-            position of the rectangle that will be divided
-        horizontal : list of :mod:`~mpl_toolkits.axes_grid.axes_size`
-            sizes for horizontal division
-        vertical : list of :mod:`~mpl_toolkits.axes_grid.axes_size`
-            sizes for vertical division
+            Position of the rectangle that will be divided.
+        horizontal : list of :mod:`~mpl_toolkits.axes_grid1.axes_size`
+            Sizes for horizontal division.
+        vertical : list of :mod:`~mpl_toolkits.axes_grid1.axes_size`
+            Sizes for vertical division.
         aspect : bool
-            if True, the overall rectangular area is reduced
-            so that the relative part of the horizontal and
-            vertical scales have the same scale.
-        anchor : {'C', 'SW', 'S', 'SE', 'E', 'NE', 'N', 'NW', 'W'}
-            placement of the reduced rectangle when *aspect* is True
+            Whether overall rectangular area is reduced so that the relative
+            part of the horizontal and vertical scales have the same scale.
+        anchor : (float, float) or {'C', 'SW', 'S', 'SE', 'E', 'NE', 'N', \
+'NW', 'W'}
+            Placement of the reduced rectangle, when *aspect* is True.
         """
 
         self._fig = fig
@@ -54,6 +49,7 @@ class Divider:
         self._horizontal = horizontal
         self._vertical = vertical
         self._anchor = anchor
+        self.set_anchor(anchor)
         self._aspect = aspect
         self._xrefindex = 0
         self._yrefindex = 0
@@ -65,13 +61,10 @@ class Divider:
     def get_vertical_sizes(self, renderer):
         return [s.get_size(renderer) for s in self.get_vertical()]
 
+    @_api.deprecated("3.5")
     def get_vsize_hsize(self):
-
-        from .axes_size import AddList
-
-        vsize = AddList(self.get_vertical())
-        hsize = AddList(self.get_horizontal())
-
+        vsize = Size.AddList(self.get_vertical())
+        hsize = Size.AddList(self.get_horizontal())
         return vsize, hsize
 
     @staticmethod
@@ -98,7 +91,7 @@ class Divider:
 
     def set_position(self, pos):
         """
-        set the position of the rectangle.
+        Set the position of the rectangle.
 
         Parameters
         ----------
@@ -108,63 +101,57 @@ class Divider:
         self._pos = pos
 
     def get_position(self):
-        "return the position of the rectangle."
+        """Return the position of the rectangle."""
         return self._pos
 
     def set_anchor(self, anchor):
         """
         Parameters
         ----------
-        anchor : {'C', 'SW', 'S', 'SE', 'E', 'NE', 'N', 'NW', 'W'}
-            anchor position
-
-          =====  ============
-          value  description
-          =====  ============
-          'C'    Center
-          'SW'   bottom left
-          'S'    bottom
-          'SE'   bottom right
-          'E'    right
-          'NE'   top right
-          'N'    top
-          'NW'   top left
-          'W'    left
-          =====  ============
+        anchor : (float, float) or {'C', 'SW', 'S', 'SE', 'E', 'NE', 'N', \
+'NW', 'W'}
+            Either an (*x*, *y*) pair of relative coordinates (0 is left or
+            bottom, 1 is right or top), 'C' (center), or a cardinal direction
+            ('SW', southwest, is bottom left, etc.).
 
+        See Also
+        --------
+        .Axes.set_anchor
         """
-        if len(anchor) != 2:
-            cbook._check_in_list(mtransforms.Bbox.coefs, anchor=anchor)
+        if isinstance(anchor, str):
+            _api.check_in_list(mtransforms.Bbox.coefs, anchor=anchor)
+        elif not isinstance(anchor, (tuple, list)) or len(anchor) != 2:
+            raise TypeError("anchor must be str or 2-tuple")
         self._anchor = anchor
 
     def get_anchor(self):
-        "return the anchor"
+        """Return the anchor."""
         return self._anchor
 
     def set_horizontal(self, h):
         """
         Parameters
         ----------
-        h : list of :mod:`~mpl_toolkits.axes_grid.axes_size`
+        h : list of :mod:`~mpl_toolkits.axes_grid1.axes_size`
             sizes for horizontal division
         """
         self._horizontal = h
 
     def get_horizontal(self):
-        "return horizontal sizes"
+        """Return horizontal sizes."""
         return self._horizontal
 
     def set_vertical(self, v):
         """
         Parameters
         ----------
-        v : list of :mod:`~mpl_toolkits.axes_grid.axes_size`
+        v : list of :mod:`~mpl_toolkits.axes_grid1.axes_size`
             sizes for vertical division
         """
         self._vertical = v
 
     def get_vertical(self):
-        "return vertical sizes"
+        """Return vertical sizes."""
         return self._vertical
 
     def set_aspect(self, aspect=False):
@@ -176,7 +163,7 @@ class Divider:
         self._aspect = aspect
 
     def get_aspect(self):
-        "return aspect"
+        """Return aspect."""
         return self._aspect
 
     def set_locator(self, _locator):
@@ -196,35 +183,34 @@ class Divider:
         Parameters
         ----------
         nx, nx1 : int
-            Integers specifying the column-position of the
-            cell. When *nx1* is None, a single *nx*-th column is
-            specified. Otherwise location of columns spanning between *nx*
-            to *nx1* (but excluding *nx1*-th column) is specified.
+            Integers specifying the column-position of the cell. When *nx1* is
+            None, a single *nx*-th column is specified. Otherwise, the
+            location of columns spanning between *nx* to *nx1* (but excluding
+            *nx1*-th column) is specified.
         ny, ny1 : int
             Same as *nx* and *nx1*, but for row positions.
         axes
         renderer
         """
 
-        figW, figH = self._fig.get_size_inches()
+        fig_w, fig_h = self._fig.bbox.size / self._fig.dpi
         x, y, w, h = self.get_position_runtime(axes, renderer)
 
         hsizes = self.get_horizontal_sizes(renderer)
         vsizes = self.get_vertical_sizes(renderer)
-        k_h = self._calc_k(hsizes, figW*w)
-        k_v = self._calc_k(vsizes, figH*h)
+        k_h = self._calc_k(hsizes, fig_w * w)
+        k_v = self._calc_k(vsizes, fig_h * h)
 
         if self.get_aspect():
             k = min(k_h, k_v)
             ox = self._calc_offsets(hsizes, k)
             oy = self._calc_offsets(vsizes, k)
 
-            ww = (ox[-1] - ox[0])/figW
-            hh = (oy[-1] - oy[0])/figH
+            ww = (ox[-1] - ox[0]) / fig_w
+            hh = (oy[-1] - oy[0]) / fig_h
             pb = mtransforms.Bbox.from_bounds(x, y, w, h)
             pb1 = mtransforms.Bbox.from_bounds(x, y, ww, hh)
-            pb1_anchored = pb1.anchored(self.get_anchor(), pb)
-            x0, y0 = pb1_anchored.x0, pb1_anchored.y0
+            x0, y0 = pb1.anchored(self.get_anchor(), pb).p0
 
         else:
             ox = self._calc_offsets(hsizes, k_h)
@@ -232,20 +218,26 @@ class Divider:
             x0, y0 = x, y
 
         if nx1 is None:
-            nx1 = nx+1
+            _api.warn_deprecated(
+                "3.5", message="Support for passing nx1=None to mean nx+1 is "
+                "deprecated since %(since)s; in a future version, nx1=None "
+                "will mean 'up to the last cell'.")
+            nx1 = nx + 1
         if ny1 is None:
-            ny1 = ny+1
+            _api.warn_deprecated(
+                "3.5", message="Support for passing ny1=None to mean ny+1 is "
+                "deprecated since %(since)s; in a future version, ny1=None "
+                "will mean 'up to the last cell'.")
+            ny1 = ny + 1
 
-        x1, w1 = x0 + ox[nx]/figW, (ox[nx1] - ox[nx])/figW
-        y1, h1 = y0 + oy[ny]/figH, (oy[ny1] - oy[ny])/figH
+        x1, w1 = x0 + ox[nx] / fig_w, (ox[nx1] - ox[nx]) / fig_w
+        y1, h1 = y0 + oy[ny] / fig_h, (oy[ny1] - oy[ny]) / fig_h
 
         return mtransforms.Bbox.from_bounds(x1, y1, w1, h1)
 
     def new_locator(self, nx, ny, nx1=None, ny1=None):
         """
-        Returns a new locator
-        (:class:`mpl_toolkits.axes_grid.axes_divider.AxesLocator`) for
-        specified cell.
+        Return a new `AxesLocator` for the specified cell.
 
         Parameters
         ----------
@@ -257,9 +249,14 @@ class Divider:
         ny, ny1 : int
             Same as *nx* and *nx1*, but for row positions.
         """
-        return AxesLocator(self, nx, ny, nx1, ny1)
+        return AxesLocator(
+            self, nx, ny,
+            nx1 if nx1 is not None else nx + 1,
+            ny1 if ny1 is not None else ny + 1)
 
     def append_size(self, position, size):
+        _api.check_in_list(["left", "right", "bottom", "top"],
+                           position=position)
         if position == "left":
             self._horizontal.insert(0, size)
             self._xrefindex += 1
@@ -268,31 +265,35 @@ class Divider:
         elif position == "bottom":
             self._vertical.insert(0, size)
             self._yrefindex += 1
-        elif position == "top":
+        else:  # 'top'
             self._vertical.append(size)
-        else:
-            cbook._check_in_list(["left", "right", "bottom", "top"],
-                                 position=position)
 
-    def add_auto_adjustable_area(self,
-                                 use_axes, pad=0.1,
-                                 adjust_dirs=None,
-                                 ):
+    def add_auto_adjustable_area(self, use_axes, pad=0.1, adjust_dirs=None):
+        """
+        Add auto-adjustable padding around *use_axes* to take their decorations
+        (title, labels, ticks, ticklabels) into account during layout.
+
+        Parameters
+        ----------
+        use_axes : `~.axes.Axes` or list of `~.axes.Axes`
+            The Axes whose decorations are taken into account.
+        pad : float, optional
+            Additional padding in inches.
+        adjust_dirs : list of {"left", "right", "bottom", "top"}, optional
+            The sides where padding is added; defaults to all four sides.
+        """
         if adjust_dirs is None:
             adjust_dirs = ["left", "right", "bottom", "top"]
-        from .axes_size import Padded, SizeFromFunc, GetExtentHelper
         for d in adjust_dirs:
-            helper = GetExtentHelper(use_axes, d)
-            size = SizeFromFunc(helper)
-            padded_size = Padded(size, pad)  # pad in inch
-            self.append_size(d, padded_size)
+            self.append_size(d, Size._AxesDecorationsSize(use_axes, d) + pad)
 
 
 class AxesLocator:
     """
-    A simple callable object, initialized with AxesDivider class,
-    returns the position and size of the given cell.
+    A callable object which returns the position and size of a given
+    AxesDivider cell.
     """
+
     def __init__(self, axes_divider, nx, ny, nx1=None, ny1=None):
         """
         Parameters
@@ -314,9 +315,17 @@ class AxesLocator:
         self._nx, self._ny = nx - _xrefindex, ny - _yrefindex
 
         if nx1 is None:
-            nx1 = nx+1
+            _api.warn_deprecated(
+                "3.5", message="Support for passing nx1=None to mean nx+1 is "
+                "deprecated since %(since)s; in a future version, nx1=None "
+                "will mean 'up to the last cell'.")
+            nx1 = nx + 1
         if ny1 is None:
-            ny1 = ny+1
+            _api.warn_deprecated(
+                "3.5", message="Support for passing ny1=None to mean ny+1 is "
+                "deprecated since %(since)s; in a future version, ny1=None "
+                "will mean 'up to the last cell'.")
+            ny1 = ny + 1
 
         self._nx1 = nx1 - _xrefindex
         self._ny1 = ny1 - _yrefindex
@@ -362,105 +371,15 @@ class SubplotDivider(Divider):
             *args* can be passed as a single 3-digit number (e.g. 234 for
             (2, 3, 4)).
         """
-
         self.figure = fig
-
-        if len(args) == 1:
-            if isinstance(args[0], SubplotSpec):
-                self._subplotspec = args[0]
-            else:
-                try:
-                    s = str(int(args[0]))
-                    rows, cols, num = map(int, s)
-                except ValueError:
-                    raise ValueError(
-                        'Single argument to subplot must be a 3-digit integer')
-                self._subplotspec = GridSpec(rows, cols)[num-1]
-                # num - 1 for converting from MATLAB to python indexing
-        elif len(args) == 3:
-            rows, cols, num = args
-            rows = int(rows)
-            cols = int(cols)
-            if isinstance(num, tuple) and len(num) == 2:
-                num = [int(n) for n in num]
-                self._subplotspec = GridSpec(rows, cols)[num[0]-1:num[1]]
-            else:
-                self._subplotspec = GridSpec(rows, cols)[int(num)-1]
-                # num - 1 for converting from MATLAB to python indexing
-        else:
-            raise ValueError(f'Illegal argument(s) to subplot: {args}')
-
-        # total = rows*cols
-        # num -= 1    # convert from matlab to python indexing
-        #             # i.e., num in range(0, total)
-        # if num >= total:
-        #     raise ValueError( 'Subplot number exceeds total subplots')
-        # self._rows = rows
-        # self._cols = cols
-        # self._num = num
-
-        # self.update_params()
-
-        # sets self.fixbox
-        self.update_params()
-
-        pos = self.figbox.bounds
-
-        Divider.__init__(self, fig, pos, horizontal or [], vertical or [],
+        super().__init__(fig, [0, 0, 1, 1],
+                         horizontal=horizontal or [], vertical=vertical or [],
                          aspect=aspect, anchor=anchor)
+        self.set_subplotspec(SubplotSpec._from_subplot_args(fig, args))
 
     def get_position(self):
-        "return the bounds of the subplot box"
-
-        self.update_params()  # update self.figbox
-        return self.figbox.bounds
-
-    # def update_params(self):
-    #     'update the subplot position from fig.subplotpars'
-
-    #     rows = self._rows
-    #     cols = self._cols
-    #     num = self._num
-
-    #     pars = self.figure.subplotpars
-    #     left = pars.left
-    #     right = pars.right
-    #     bottom = pars.bottom
-    #     top = pars.top
-    #     wspace = pars.wspace
-    #     hspace = pars.hspace
-    #     totWidth = right-left
-    #     totHeight = top-bottom
-
-    #     figH = totHeight/(rows + hspace*(rows-1))
-    #     sepH = hspace*figH
-
-    #     figW = totWidth/(cols + wspace*(cols-1))
-    #     sepW = wspace*figW
-
-    #     rowNum, colNum =  divmod(num, cols)
-
-    #     figBottom = top - (rowNum+1)*figH - rowNum*sepH
-    #     figLeft = left + colNum*(figW + sepW)
-
-    #     self.figbox = mtransforms.Bbox.from_bounds(figLeft, figBottom,
-    #                                                figW, figH)
-
-    def update_params(self):
-        """Update the subplot position from fig.subplotpars."""
-        self.figbox = self.get_subplotspec().get_position(self.figure)
-
-    def get_geometry(self):
-        """Get the subplot geometry, e.g., (2, 2, 3)."""
-        rows, cols, num1, num2 = self.get_subplotspec().get_geometry()
-        return rows, cols, num1+1  # for compatibility
-
-    # COVERAGE NOTE: Never used internally or from examples
-    def change_geometry(self, numrows, numcols, num):
-        """Change subplot geometry, e.g., from (1, 1, 1) to (2, 2, 3)."""
-        self._subplotspec = GridSpec(numrows, numcols)[num-1]
-        self.update_params()
-        self.set_position(self.figbox)
+        """Return the bounds of the subplot box."""
+        return self.get_subplotspec().get_position(self.figure).bounds
 
     def get_subplotspec(self):
         """Get the SubplotSpec instance."""
@@ -469,11 +388,12 @@ class SubplotDivider(Divider):
     def set_subplotspec(self, subplotspec):
         """Set the SubplotSpec instance."""
         self._subplotspec = subplotspec
+        self.set_position(subplotspec.get_position(self.figure))
 
 
 class AxesDivider(Divider):
     """
-    Divider based on the pre-existing axes.
+    Divider based on the preexisting axes.
     """
 
     def __init__(self, axes, xref=None, yref=None):
@@ -494,7 +414,7 @@ class AxesDivider(Divider):
         else:
             self._yref = yref
 
-        Divider.__init__(self, fig=axes.get_figure(), pos=None,
+        super().__init__(fig=axes.get_figure(), pos=None,
                          horizontal=[self._xref], vertical=[self._yref],
                          aspect=None, anchor="C")
 
@@ -510,31 +430,14 @@ class AxesDivider(Divider):
 
     def new_horizontal(self, size, pad=None, pack_start=False, **kwargs):
         """
-        Add a new axes on the right (or left) side of the main axes.
+        Helper method for ``append_axes("left")`` and ``append_axes("right")``.
 
-        Parameters
-        ----------
-        size : :mod:`~mpl_toolkits.axes_grid.axes_size` or float or str
-            A width of the axes. If float or string is given, *from_any*
-            function is used to create the size, with *ref_size* set to AxesX
-            instance of the current axes.
-        pad : :mod:`~mpl_toolkits.axes_grid.axes_size` or float or str
-            Pad between the axes. It takes same argument as *size*.
-        pack_start : bool
-            If False, the new axes is appended at the end
-            of the list, i.e., it became the right-most axes. If True, it is
-            inserted at the start of the list, and becomes the left-most axes.
-        **kwargs
-            All extra keywords arguments are passed to the created axes.
-            If *axes_class* is given, the new axes will be created as an
-            instance of the given class. Otherwise, the same class of the
-            main axes will be used.
+        See the documentation of `append_axes` for more details.
+
+        :meta private:
         """
         if pad is None:
-            cbook.warn_deprecated(
-                "3.2", message="In a future version, 'pad' will default to "
-                "rcParams['figure.subplot.wspace'].  Set pad=0 to keep the "
-                "old behavior.")
+            pad = mpl.rcParams["figure.subplot.wspace"] * self._xref
         if pad:
             if not isinstance(pad, Size._Base):
                 pad = Size.from_any(pad, fraction_ref=self._xref)
@@ -559,31 +462,14 @@ class AxesDivider(Divider):
 
     def new_vertical(self, size, pad=None, pack_start=False, **kwargs):
         """
-        Add a new axes on the top (or bottom) side of the main axes.
+        Helper method for ``append_axes("top")`` and ``append_axes("bottom")``.
 
-        Parameters
-        ----------
-        size : :mod:`~mpl_toolkits.axes_grid.axes_size` or float or str
-            A height of the axes. If float or string is given, *from_any*
-            function is used to create the size, with *ref_size* set to AxesX
-            instance of the current axes.
-        pad : :mod:`~mpl_toolkits.axes_grid.axes_size` or float or str
-            Pad between the axes. It takes same argument as *size*.
-        pack_start : bool
-            If False, the new axes is appended at the end
-            of the list, i.e., it became the right-most axes. If True, it is
-            inserted at the start of the list, and becomes the left-most axes.
-        **kwargs
-            All extra keywords arguments are passed to the created axes.
-            If *axes_class* is given, the new axes will be created as an
-            instance of the given class. Otherwise, the same class of the
-            main axes will be used.
+        See the documentation of `append_axes` for more details.
+
+        :meta private:
         """
         if pad is None:
-            cbook.warn_deprecated(
-                "3.2", message="In a future version, 'pad' will default to "
-                "rcParams['figure.subplot.hspace'].  Set pad=0 to keep the "
-                "old behavior.")
+            pad = mpl.rcParams["figure.subplot.hspace"] * self._yref
         if pad:
             if not isinstance(pad, Size._Base):
                 pad = Size.from_any(pad, fraction_ref=self._yref)
@@ -601,33 +487,43 @@ class AxesDivider(Divider):
         else:
             self._vertical.append(size)
             locator = self.new_locator(
-                nx=self._xrefindex, ny=len(self._vertical)-1)
+                nx=self._xrefindex, ny=len(self._vertical) - 1)
         ax = self._get_new_axes(**kwargs)
         ax.set_axes_locator(locator)
         return ax
 
-    def append_axes(self, position, size, pad=None, add_to_figure=True,
-                    **kwargs):
+    @_api.delete_parameter("3.5", "add_to_figure", alternative="ax.remove()")
+    def append_axes(self, position, size, pad=None, add_to_figure=True, *,
+                    axes_class=None, **kwargs):
         """
-        Create an axes at the given *position* with the same height
-        (or width) of the main axes.
+        Add a new axes on a given side of the main axes.
 
-         *position*
-           ["left"|"right"|"bottom"|"top"]
-
-         *size* and *pad* should be axes_grid.axes_size compatible.
+        Parameters
+        ----------
+        position : {"left", "right", "bottom", "top"}
+            Where the new axes is positioned relative to the main axes.
+        size : :mod:`~mpl_toolkits.axes_grid1.axes_size` or float or str
+            The axes width or height.  float or str arguments are interpreted
+            as ``axes_size.from_any(size, AxesX(<main_axes>))`` for left or
+            right axes, and likewise with ``AxesY`` for bottom or top axes.
+        pad : :mod:`~mpl_toolkits.axes_grid1.axes_size` or float or str
+            Padding between the axes.  float or str arguments are interpreted
+            as for *size*.  Defaults to :rc:`figure.subplot.wspace` times the
+            main Axes width (left or right axes) or :rc:`figure.subplot.hspace`
+            times the main Axes height (bottom or top axes).
+        axes_class : subclass type of `~.axes.Axes`, optional
+            The type of the new axes.  Defaults to the type of the main axes.
+        **kwargs
+            All extra keywords arguments are passed to the created axes.
         """
-        if position == "left":
-            ax = self.new_horizontal(size, pad, pack_start=True, **kwargs)
-        elif position == "right":
-            ax = self.new_horizontal(size, pad, pack_start=False, **kwargs)
-        elif position == "bottom":
-            ax = self.new_vertical(size, pad, pack_start=True, **kwargs)
-        elif position == "top":
-            ax = self.new_vertical(size, pad, pack_start=False, **kwargs)
-        else:
-            cbook._check_in_list(["left", "right", "bottom", "top"],
-                                 position=position)
+        create_axes, pack_start = _api.check_getitem({
+            "left": (self.new_horizontal, True),
+            "right": (self.new_horizontal, False),
+            "bottom": (self.new_vertical, True),
+            "top": (self.new_vertical, False),
+        }, position=position)
+        ax = create_axes(
+            size, pad, pack_start=pack_start, axes_class=axes_class, **kwargs)
         if add_to_figure:
             self._fig.add_axes(ax)
         return ax
@@ -662,51 +558,70 @@ class AxesDivider(Divider):
             return None
 
 
+# Helper for HBoxDivider/VBoxDivider.
+# The variable names are written for a horizontal layout, but the calculations
+# work identically for vertical layouts (and likewise for the helpers below).
+def _determine_karray(summed_widths, equal_heights, total_width, max_height):
+    n = len(equal_heights)
+    eq_rs, eq_as = np.asarray(equal_heights).T
+    sm_rs, sm_as = np.asarray(summed_widths).T
+    A = np.zeros((n + 1, n + 1))
+    B = np.zeros(n + 1)
+    np.fill_diagonal(A[:n, :n], eq_rs)
+    A[:n, -1] = -1
+    A[-1, :-1] = sm_rs
+    B[:n] = -eq_as
+    B[-1] = total_width - sum(sm_as)
+    # A @ K = B: This solves for {k_0, ..., k_{N-1}, H} so that
+    #   eq_r_i * k_i + eq_a_i = H for all i: all axes have the same height
+    #   sum(sm_r_i * k_i + sm_a_i) = total_summed_width: fixed total width
+    # (foo_r_i * k_i + foo_a_i will end up being the size of foo.)
+    karray_and_height = np.linalg.solve(A, B)
+    karray = karray_and_height[:-1]
+    height = karray_and_height[-1]
+    if height > max_height:  # Additionally, upper-bound the height.
+        karray = (max_height - eq_as) / eq_rs
+    return karray
+
+
+# Helper for HBoxDivider/VBoxDivider (see above re: variable naming).
+def _calc_offsets(summed_sizes, karray):
+    offsets = [0.]
+    for (r, a), k in zip(summed_sizes, karray):
+        offsets.append(offsets[-1] + r*k + a)
+    return offsets
+
+
+# Helper for HBoxDivider/VBoxDivider (see above re: variable naming).
+def _locate(x, y, w, h, summed_widths, equal_heights, fig_w, fig_h, anchor):
+    karray = _determine_karray(
+        summed_widths, equal_heights,
+        total_width=fig_w * w, max_height=fig_h * h)
+    ox = _calc_offsets(summed_widths, karray)
+
+    ww = (ox[-1] - ox[0]) / fig_w
+    h0_r, h0_a = equal_heights[0]
+    hh = (karray[0]*h0_r + h0_a) / fig_h
+    pb = mtransforms.Bbox.from_bounds(x, y, w, h)
+    pb1 = mtransforms.Bbox.from_bounds(x, y, ww, hh)
+    x0, y0 = pb1.anchored(anchor, pb).p0
+
+    return x0, y0, ox, hh
+
+
 class HBoxDivider(SubplotDivider):
+    """
+    A `SubplotDivider` for laying out axes horizontally, while ensuring that
+    they have equal heights.
 
-    def __init__(self, fig, *args, **kwargs):
-        SubplotDivider.__init__(self, fig, *args, **kwargs)
-
-    @staticmethod
-    def _determine_karray(equivalent_sizes, appended_sizes,
-                          max_equivalent_size,
-                          total_appended_size):
-
-        n = len(equivalent_sizes)
-        import numpy as np
-        A = np.mat(np.zeros((n+1, n+1), dtype="d"))
-        B = np.zeros((n+1), dtype="d")
-        # AxK = B
-
-        # populated A
-        for i, (r, a) in enumerate(equivalent_sizes):
-            A[i, i] = r
-            A[i, -1] = -1
-            B[i] = -a
-        A[-1, :-1] = [r for r, a in appended_sizes]
-        B[-1] = total_appended_size - sum([a for rs, a in appended_sizes])
-
-        karray_H = (A.I*np.mat(B).T).A1
-        karray = karray_H[:-1]
-        H = karray_H[-1]
-
-        if H > max_equivalent_size:
-            karray = ((max_equivalent_size -
-                      np.array([a for r, a in equivalent_sizes]))
-                      / np.array([r for r, a in equivalent_sizes]))
-        return karray
-
-    @staticmethod
-    def _calc_offsets(appended_sizes, karray):
-        offsets = [0.]
-        for (r, a), k in zip(appended_sizes, karray):
-            offsets.append(offsets[-1] + r*k + a)
-        return offsets
+    Examples
+    --------
+    .. plot:: gallery/axes_grid1/demo_axes_hbox_divider.py
+    """
 
     def new_locator(self, nx, nx1=None):
         """
-        Create a new `~mpl_toolkits.axes_grid.axes_divider.AxesLocator` for
-        the specified cell.
+        Create a new `AxesLocator` for the specified cell.
 
         Parameters
         ----------
@@ -715,90 +630,37 @@ class HBoxDivider(SubplotDivider):
             cell. When *nx1* is None, a single *nx*-th column is
             specified. Otherwise location of columns spanning between *nx*
             to *nx1* (but excluding *nx1*-th column) is specified.
-        ny, ny1 : int
-            Same as *nx* and *nx1*, but for row positions.
         """
-        return AxesLocator(self, nx, 0, nx1, None)
-
-    def _locate(self, x, y, w, h,
-                y_equivalent_sizes, x_appended_sizes,
-                figW, figH):
-        """
-        Parameters
-        ----------
-        x
-        y
-        w
-        h
-        y_equivalent_sizes
-        x_appended_sizes
-        figW
-        figH
-        """
-
-        equivalent_sizes = y_equivalent_sizes
-        appended_sizes = x_appended_sizes
-
-        max_equivalent_size = figH*h
-        total_appended_size = figW*w
-        karray = self._determine_karray(equivalent_sizes, appended_sizes,
-                                        max_equivalent_size,
-                                        total_appended_size)
-
-        ox = self._calc_offsets(appended_sizes, karray)
-
-        ww = (ox[-1] - ox[0])/figW
-        ref_h = equivalent_sizes[0]
-        hh = (karray[0]*ref_h[0] + ref_h[1])/figH
-        pb = mtransforms.Bbox.from_bounds(x, y, w, h)
-        pb1 = mtransforms.Bbox.from_bounds(x, y, ww, hh)
-        pb1_anchored = pb1.anchored(self.get_anchor(), pb)
-        x0, y0 = pb1_anchored.x0, pb1_anchored.y0
-
-        return x0, y0, ox, hh
+        return AxesLocator(self, nx, 0, nx1 if nx1 is not None else nx + 1, 1)
 
     def locate(self, nx, ny, nx1=None, ny1=None, axes=None, renderer=None):
-        """
-        Parameters
-        ----------
-        axes_divider : AxesDivider
-        nx, nx1 : int
-            Integers specifying the column-position of the
-            cell. When *nx1* is None, a single *nx*-th column is
-            specified. Otherwise location of columns spanning between *nx*
-            to *nx1* (but excluding *nx1*-th column) is specified.
-        ny, ny1 : int
-            Same as *nx* and *nx1*, but for row positions.
-        axes
-        renderer
-        """
-
-        figW, figH = self._fig.get_size_inches()
+        # docstring inherited
+        fig_w, fig_h = self._fig.bbox.size / self._fig.dpi
         x, y, w, h = self.get_position_runtime(axes, renderer)
-
-        y_equivalent_sizes = self.get_vertical_sizes(renderer)
-        x_appended_sizes = self.get_horizontal_sizes(renderer)
-        x0, y0, ox, hh = self._locate(x, y, w, h,
-                                      y_equivalent_sizes, x_appended_sizes,
-                                      figW, figH)
+        summed_ws = self.get_horizontal_sizes(renderer)
+        equal_hs = self.get_vertical_sizes(renderer)
+        x0, y0, ox, hh = _locate(
+            x, y, w, h, summed_ws, equal_hs, fig_w, fig_h, self.get_anchor())
         if nx1 is None:
-            nx1 = nx+1
-
-        x1, w1 = x0 + ox[nx]/figW, (ox[nx1] - ox[nx])/figW
+            _api.warn_deprecated(
+                "3.5", message="Support for passing nx1=None to mean nx+1 is "
+                "deprecated since %(since)s; in a future version, nx1=None "
+                "will mean 'up to the last cell'.")
+            nx1 = nx + 1
+        x1, w1 = x0 + ox[nx] / fig_w, (ox[nx1] - ox[nx]) / fig_w
         y1, h1 = y0, hh
-
         return mtransforms.Bbox.from_bounds(x1, y1, w1, h1)
 
 
-class VBoxDivider(HBoxDivider):
+class VBoxDivider(SubplotDivider):
     """
-    The Divider class whose rectangle area is specified as a subplot geometry.
+    A `SubplotDivider` for laying out axes vertically, while ensuring that they
+    have equal widths.
     """
 
     def new_locator(self, ny, ny1=None):
         """
-        Create a new `~mpl_toolkits.axes_grid.axes_divider.AxesLocator` for
-        the specified cell.
+        Create a new `AxesLocator` for the specified cell.
 
         Parameters
         ----------
@@ -808,39 +670,24 @@ class VBoxDivider(HBoxDivider):
             specified. Otherwise location of rows spanning between *ny*
             to *ny1* (but excluding *ny1*-th row) is specified.
         """
-        return AxesLocator(self, 0, ny, None, ny1)
+        return AxesLocator(self, 0, ny, 1, ny1 if ny1 is not None else ny + 1)
 
     def locate(self, nx, ny, nx1=None, ny1=None, axes=None, renderer=None):
-        """
-        Parameters
-        ----------
-        axes_divider : AxesDivider
-        nx, nx1 : int
-            Integers specifying the column-position of the
-            cell. When *nx1* is None, a single *nx*-th column is
-            specified. Otherwise location of columns spanning between *nx*
-            to *nx1* (but excluding *nx1*-th column) is specified.
-        ny, ny1 : int
-            Same as *nx* and *nx1*, but for row positions.
-        axes
-        renderer
-        """
-
-        figW, figH = self._fig.get_size_inches()
+        # docstring inherited
+        fig_w, fig_h = self._fig.bbox.size / self._fig.dpi
         x, y, w, h = self.get_position_runtime(axes, renderer)
-
-        x_equivalent_sizes = self.get_horizontal_sizes(renderer)
-        y_appended_sizes = self.get_vertical_sizes(renderer)
-
-        y0, x0, oy, ww = self._locate(y, x, h, w,
-                                      x_equivalent_sizes, y_appended_sizes,
-                                      figH, figW)
+        summed_hs = self.get_vertical_sizes(renderer)
+        equal_ws = self.get_horizontal_sizes(renderer)
+        y0, x0, oy, ww = _locate(
+            y, x, h, w, summed_hs, equal_ws, fig_h, fig_w, self.get_anchor())
         if ny1 is None:
-            ny1 = ny+1
-
+            _api.warn_deprecated(
+                "3.5", message="Support for passing ny1=None to mean ny+1 is "
+                "deprecated since %(since)s; in a future version, ny1=None "
+                "will mean 'up to the last cell'.")
+            ny1 = ny + 1
         x1, w1 = x0, ww
-        y1, h1 = y0 + oy[ny]/figH, (oy[ny1] - oy[ny])/figH
-
+        y1, h1 = y0 + oy[ny] / fig_h, (oy[ny1] - oy[ny]) / fig_h
         return mtransforms.Bbox.from_bounds(x1, y1, w1, h1)
 
 
@@ -852,15 +699,20 @@ def make_axes_locatable(axes):
     return divider
 
 
-def make_axes_area_auto_adjustable(ax,
-                                   use_axes=None, pad=0.1,
-                                   adjust_dirs=None):
+def make_axes_area_auto_adjustable(
+        ax, use_axes=None, pad=0.1, adjust_dirs=None):
+    """
+    Add auto-adjustable padding around *ax* to take its decorations (title,
+    labels, ticks, ticklabels) into account during layout, using
+    `Divider.add_auto_adjustable_area`.
+
+    By default, padding is determined from the decorations of *ax*.
+    Pass *use_axes* to consider the decorations of other Axes instead.
+    """
     if adjust_dirs is None:
         adjust_dirs = ["left", "right", "bottom", "top"]
     divider = make_axes_locatable(ax)
-
     if use_axes is None:
         use_axes = ax
-
     divider.add_auto_adjustable_area(use_axes=use_axes, pad=pad,
                                      adjust_dirs=adjust_dirs)
