@@ -1,20 +1,19 @@
 """
-==================
-Demo Edge Colorbar
-==================
+===============================
+Per-row or per-column colorbars
+===============================
 
 This example shows how to use one common colorbar for each row or column
 of an image grid.
 """
+
+from matplotlib import cbook
 import matplotlib.pyplot as plt
 from mpl_toolkits.axes_grid1 import AxesGrid
 
 
 def get_demo_image():
-    import numpy as np
-    from matplotlib.cbook import get_sample_data
-    f = get_sample_data("axes_grid/bivariate_normal.npy", asfileobj=False)
-    z = np.load(f)
+    z = cbook.get_sample_data("axes_grid/bivariate_normal.npy", np_load=True)
     # z is a numpy array of 15x15
     return z, (-3, 4, -4, 3)
 
@@ -36,10 +35,9 @@ def demo_bottom_cbar(fig):
                     )
 
     Z, extent = get_demo_image()
-    cmaps = [plt.get_cmap("autumn"), plt.get_cmap("summer")]
+    cmaps = ["autumn", "summer"]
     for i in range(4):
-        im = grid[i].imshow(Z, extent=extent, interpolation="nearest",
-                            cmap=cmaps[i//2])
+        im = grid[i].imshow(Z, extent=extent, cmap=cmaps[i//2])
         if i % 2:
             grid.cbar_axes[i//2].colorbar(im)
 
@@ -67,10 +65,9 @@ def demo_right_cbar(fig):
                     cbar_pad="2%",
                     )
     Z, extent = get_demo_image()
-    cmaps = [plt.get_cmap("spring"), plt.get_cmap("winter")]
+    cmaps = ["spring", "winter"]
     for i in range(4):
-        im = grid[i].imshow(Z, extent=extent, interpolation="nearest",
-                            cmap=cmaps[i//2])
+        im = grid[i].imshow(Z, extent=extent, cmap=cmaps[i//2])
         if i % 2:
             grid.cbar_axes[i//2].colorbar(im)
 
