@@ -6,28 +6,39 @@ Pick Event Demo2
 Compute the mean (mu) and standard deviation (sigma) of 100 data sets and plot
 mu vs. sigma.  When you click on one of the (mu, sigma) points, plot the raw
 data from the dataset that generated this point.
+
+.. note::
+    This example exercises the interactive capabilities of Matplotlib, and this
+    will not appear in the static documentation. Please run this code on your
+    machine to see the interactivity.
+
+    You can copy and paste individual parts, or download the entire example
+    using the link at the bottom of the page.
 """
 import numpy as np
 import matplotlib.pyplot as plt
 
 
+# Fixing random state for reproducibility
+np.random.seed(19680801)
+
 X = np.random.rand(100, 1000)
 xs = np.mean(X, axis=1)
 ys = np.std(X, axis=1)
 
 fig, ax = plt.subplots()
 ax.set_title('click on point to plot time series')
-line, = ax.plot(xs, ys, 'o', picker=5)  # 5 points tolerance
+line, = ax.plot(xs, ys, 'o', picker=True, pickradius=5)
 
 
 def onpick(event):
 
     if event.artist != line:
-        return True
+        return
 
     N = len(event.ind)
     if not N:
-        return True
+        return
 
     figi, axs = plt.subplots(N, squeeze=False)
     for ax, dataind in zip(axs.flat, event.ind):
@@ -36,7 +47,7 @@ def onpick(event):
                 transform=ax.transAxes, va='top')
         ax.set_ylim(-0.5, 1.5)
     figi.show()
-    return True
+
 
 fig.canvas.mpl_connect('pick_event', onpick)
 
