@@ -16,20 +16,15 @@ plt.title(r'$\alpha_i > \beta_i$', fontsize=20)
 plt.text(1, -0.6, r'$\sum_{i=0}^\infty x_i$', fontsize=20)
 plt.text(0.6, 0.6, r'$\mathcal{A}\mathrm{sin}(2 \omega t)$',
          fontsize=20)
-plt.xlabel('time (s)')
-plt.ylabel('volts (mV)')
+plt.xlabel('Time [s]')
+plt.ylabel('Voltage [mV]')
 plt.show()
 
 #############################################################################
 #
-# ------------
+# .. admonition:: References
 #
-# References
-# """"""""""
+#    The use of the following functions, methods, classes and modules is shown
+#    in this example:
 #
-# The use of the following functions, methods, classes and modules is shown
-# in this example:
-
-import matplotlib
-matplotlib.pyplot.text
-matplotlib.axes.Axes.text
+#    - `matplotlib.axes.Axes.text` / `matplotlib.pyplot.text`
