@@ -6,6 +6,10 @@ Nested Gridspecs
 GridSpecs can be nested, so that a subplot from a parent GridSpec can
 set the position for a nested grid of subplots.
 
+Note that the same functionality can be achieved more directly with
+`~.FigureBase.subfigures`; see
+:doc:`/gallery/subplots_axes_and_figures/subfigures`.
+
 """
 import matplotlib.pyplot as plt
 import matplotlib.gridspec as gridspec
@@ -18,24 +22,24 @@ def format_axes(fig):
 
 
 # gridspec inside gridspec
-f = plt.figure()
+fig = plt.figure()
 
-gs0 = gridspec.GridSpec(1, 2, figure=f)
+gs0 = gridspec.GridSpec(1, 2, figure=fig)
 
 gs00 = gridspec.GridSpecFromSubplotSpec(3, 3, subplot_spec=gs0[0])
 
-ax1 = f.add_subplot(gs00[:-1, :])
-ax2 = f.add_subplot(gs00[-1, :-1])
-ax3 = f.add_subplot(gs00[-1, -1])
+ax1 = fig.add_subplot(gs00[:-1, :])
+ax2 = fig.add_subplot(gs00[-1, :-1])
+ax3 = fig.add_subplot(gs00[-1, -1])
 
 # the following syntax does the same as the GridSpecFromSubplotSpec call above:
 gs01 = gs0[1].subgridspec(3, 3)
 
-ax4 = f.add_subplot(gs01[:, :-1])
-ax5 = f.add_subplot(gs01[:-1, -1])
-ax6 = f.add_subplot(gs01[-1, -1])
+ax4 = fig.add_subplot(gs01[:, :-1])
+ax5 = fig.add_subplot(gs01[:-1, -1])
+ax6 = fig.add_subplot(gs01[-1, -1])
 
 plt.suptitle("GridSpec Inside GridSpec")
-format_axes(f)
+format_axes(fig)
 
 plt.show()
